package ImageTypeOptimizer::CMS;

use strict;

use LWP::UserAgent;
use Image::Size;

sub add_source {
    my ($cb, $app, $template) = @_;

    my $old = <<HERE;
    <mt:if name="request.saved">
HERE
    $old = quotemeta($old);

    my $plugin = $app->component('ImageTypeOptimizer');
    my $message = $plugin->translate('Your changes have been optimized.');

    my $new = <<HERE;
    <mt:if name="request.optimized">
        <mtapp:statusmsg
            id="optimized"
            class="success">
            $message
        </mtapp:statusmsg>
    </mt:if>
    <mt:if name="request.saved">
HERE
    $$template =~ s/$old/$new/;
}

sub optimize_image_type {
    my $app = shift;

    my $blog_id = $app->blog->id;
    my @ids = $app->param('id');

    my $plugin = $app->component('ImageTypeOptimizer');
    my $outlog = $plugin->get_config_value('outlog', 'blog:'.$app->blog->id);

    for my $id (@ids) {
        my $entry = MT::Entry->load({ id => $id, blog_id  => $blog_id });
        _change_extension($app, $entry, $outlog);
    }
    $app->add_return_arg( 'optimized' => 1 );
    $app->call_return;
}

sub _change_extension {
    my ($app, $entry, $outlog) = @_;

    eval { require Image::Magick; };
    if ($@) {
        warn "$@";
        return;
    }

    my @result;
    my $text = $entry->text;
    if ($text) {
        @result = _get_result($app, $text, $entry, $outlog);
        $entry->text(join("\n", @result));
    }
    my $text_more = $entry->text_more;
    if ($text_more) {
        @result = _get_result($app, $text_more, $entry, $outlog);
        $entry->text_more(join("\n", @result));
    }
    $entry->save;
}

sub _get_result {
    my ($app, $text, $entry, $outlog) = @_;
    $text =~ s!(<\s*img\s+[^>]*)src="([^"]*)"([^>]*/?>)!
        $1 . 'src="'. _change($app, $2, $entry, $outlog) . '" '. $3
    !ige;
    $text;
}

sub _change {
    my ($app, $src1, $entry, $outlog) = @_;
    return $src1 if $src1 !~ /\.png$|\.jpe?g$/;

    my ($src2, $size1, $size2);
    my $ua = LWP::UserAgent->new;
    my $req1 = HTTP::Request->new(GET => $src1);
    my $res1 = $ua->request($req1);
    return $src1 if !$res1->is_success;

    my $img1 = Image::Magick->new;
    $img1->BlobToImage($res1->content);
    $size1 = $img1->Get("filesize");

    (my $ext = $src1) =~ s/.*\.([^\.]+)$/$1/;
    if ( $ext eq 'png') {
        ($src2 = $src1) =~ s/\.png/.jpg/;
    } elsif ( $ext =~ /^jpe?g$/ ) {
        ($src2 = $src1) =~ s/\.jpe?g/.png/;
    }
    my $req2 = HTTP::Request->new(GET => $src2);
    my $res2 = $ua->request($req2);
    return $src1 if !$res2->is_success;

    my $img2 = Image::Magick->new;
    $img2->BlobToImage($res2->content);
    $size2 = $img2->Get("filesize");
    if ( ($size1 > 0 && $size2 > 0) && $size1 > $size2 ) {
        $app->log($entry->title.'('.$entry->id.')'."src changed to '$src2' from '$src1'.") if $outlog;
        return $src2;
    }
    return $src1;
}

1;
